/***************************************************************************\
  "global_conv.h" - Global Conversion Functions (Global Phase)
  Revision 3; 10 Nov 2002 - 14 Nov 2002
  Copyright (C) 2002  Kevin P. Albrecht
\***************************************************************************/

#ifndef GLOBAL_CONV_H
#define GLOBAL_CONV_H

/////////////////////////
// DEFINES //////////////
/////////////////////////

// External Defines //
// GLOBAL_MANTISSA_SIZE

/////////////////////////
// INCLUDES /////////////
/////////////////////////

// Standard //

// Local //
#include "k_string.h"

/////////////////////////
// DECLARATIONS /////////
/////////////////////////

char *conv_bs (bool ab);
char *conv_is (int an);
char *conv_fs (float af);
char *conv_cs (char ac);
bool conv_sb (char *aS);
bool conv_sbD (char *aS);
int conv_si (char *aS);
int conv_siD (char *aS);
float conv_sf (char *aS);
float conv_sfD (char *aS);

float *k_new_f (float f);
int *k_new_n (int n);

/////////////////////////
// DEFINITIONS //////////
/////////////////////////

char *conv_bs (bool ab)
{
  if (ab) return "1";
  else    return "0";
}

char *conv_is (int an)
{
  return k_int (an);
}

char *conv_fs (float af)
{
  return k_float (af, GLOBAL_MANTISSA_SIZE);
}

char *conv_cs (char ac)
{
  char *s = k_new (2);
  s[0] = ac;
  s[1] = '\0';
  return s;
}

bool conv_sb (char *aS)
{
  if (aS == NULL) return false;
  return (k_same (aS, "false") || k_same (aS, "0") || k_same (aS, "0.") ||
          k_same (aS, "0.0") || k_same (aS, "0.00") || k_same (aS, "0.000") ||
          k_same (aS, "0.0000") || k_same (aS, "0.00000")
         ) ? false : true;
}

bool conv_sbD (char *aS)
{
  bool temp = conv_sb (aS);
  k_del (aS);
  return temp;
}

int conv_si (char *aS)
{
  if (aS == NULL) return 0;
  return (atoi (aS));
}

int conv_siD (char *aS)
{
  int temp = conv_si (aS);
  k_del (aS);
  return temp;
}

float conv_sf (char *aS)
{
  if (aS == NULL) return 0.0;
  return ((float) atof (aS));
}

float conv_sfD (char *aS)
{
  float temp = conv_sf (aS);
  k_del (aS);
  return temp;
}

float *k_new_f (float f)
{
  float *pf;
  pf = (float *) malloc (sizeof(float));
  *pf = f;
  return pf;
}

int *k_new_n (int n)
{
  int *pn;
  pn = (int *) malloc (sizeof (int));
  *pn = n;
  return pn;
}


#endif
